﻿using System;
using System.Windows;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 색상 셀
    /// </summary>
    public class ColorCell : FrameworkElement
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 선택 여부 속성
        /// </summary>
        public static readonly DependencyProperty IsSelectedProperty;

        /// <summary>
        /// 하이라이트 여부 속성
        /// </summary>
        public static readonly DependencyProperty IsHighlightedProperty;

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 셀 크기
        /// </summary>
        private static readonly Size _cellSize = new Size(20, 20);

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 색상 드로잉 비주얼
        /// </summary>
        private DrawingVisual colorDrawingVisual;

        /// <summary>
        /// 브러시
        /// </summary>
        private Brush brush;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 선택 여부 - IsSelected

        /// <summary>
        /// 선택 여부
        /// </summary>
        public bool IsSelected
        {
            set
            {
                SetValue(IsSelectedProperty, value);
            }
            get
            {
                return (bool)GetValue(IsSelectedProperty);
            }
        }

        #endregion
        #region 하이라이트 여부 - IsHighlighted

        /// <summary>
        /// 하이라이트 여부
        /// </summary>
        public bool IsHighlighted
        {
            set
            {
                SetValue(IsHighlightedProperty, value);
            }
            get
            {
                return (bool)GetValue(IsHighlightedProperty);
            }
        }

        #endregion
        #region 브러시 - Brush

        /// <summary>
        /// 브러시
        /// </summary>
        public Brush Brush
        {
            get
            {
                return this.brush;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 비주얼 자식 수 - VisualChildrenCount

        /// <summary>
        /// 비주얼 자식 수
        /// </summary>
        protected override int VisualChildrenCount
        {
            get
            {
                return 1;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - ColorCell()

        /// <summary>
        /// 생성자
        /// </summary>
        static ColorCell()
        {
            IsSelectedProperty = DependencyProperty.Register
            (
                "IsSelected",
                typeof(bool),
                typeof(ColorCell),
                new FrameworkPropertyMetadata
                (
                    false,
                    FrameworkPropertyMetadataOptions.AffectsRender
                )
            );

            IsHighlightedProperty = DependencyProperty.Register
            (
                "IsHighlighted",
                typeof(bool),
                typeof(ColorCell),
                new FrameworkPropertyMetadata
                (
                    false,
                    FrameworkPropertyMetadataOptions.AffectsRender
                )
            );
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ColorCell(color)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="color">색상</param>
        public ColorCell(Color color)
        {
            this.colorDrawingVisual = new DrawingVisual();

            DrawingContext drawingContext = this.colorDrawingVisual.RenderOpen();

            Rect rect = new Rect(new Point(0, 0), _cellSize);

            rect.Inflate(-4, -4);

            Pen pen = new Pen(SystemColors.ControlTextBrush, 1);

            this.brush = new SolidColorBrush(color);

            drawingContext.DrawRectangle(this.brush, pen, rect);

            drawingContext.Close();

            AddVisualChild(this.colorDrawingVisual);

            AddLogicalChild(this.colorDrawingVisual);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Method
        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 비주얼 자식 구하기 - GetVisualChild(index)

        /// <summary>
        /// 비주얼 자식 구하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <returns>비주얼 자식</returns>
        protected override Visual GetVisualChild(int index)
        {
            if(index > 0)
            {
                throw new ArgumentOutOfRangeException("index");
            }

            return this.colorDrawingVisual;
        }

        #endregion
        #region 측정하기 (오버라이딩) - MeasureOverride(availableSize)

        /// <summary>
        /// 측정하기 (오버라이딩)
        /// </summary>
        /// <param name="availableSize">이용 가능한 크기</param>
        /// <returns>측정 크기</returns>
        protected override Size MeasureOverride(Size availableSize)
        {
            return _cellSize;
        }

        #endregion
        #region 렌더링 처리하기 - OnRender(drawingContext)

        /// <summary>
        /// 렌더링 처리하기
        /// </summary>
        /// <param name="drawingContext">DrawingContext</param>
        protected override void OnRender(DrawingContext drawingContext)
        {
            Rect rect = new Rect(new Point(0, 0), RenderSize);

            rect.Inflate(-1, -1);

            Pen pen = new Pen(SystemColors.HighlightBrush, 1);

            if(IsHighlighted)
            {
                drawingContext.DrawRectangle(SystemColors.ControlDarkBrush, pen, rect);
            }
            else if(IsSelected)
            {
                drawingContext.DrawRectangle(SystemColors.ControlLightBrush, pen, rect);
            }
            else
            {
                drawingContext.DrawRectangle(Brushes.Transparent, null, rect);
            }
        }

        #endregion
    }
}