﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            SizeToContent = SizeToContent.WidthAndHeight;
            Padding       = new Thickness(10);
            FontFamily    = new FontFamily("나눔고딕코딩");
            FontSize      = 16;
            Title         = "Control 클래스 : 색상 그리드 사용하기";

            StackPanel stackPanel = new StackPanel();

            stackPanel.Orientation = Orientation.Horizontal;

            Content = stackPanel;

            Button button1 = new Button();

            button1.Content             = "테스트 버튼";
            button1.Margin              = new Thickness(24);
            button1.HorizontalAlignment = HorizontalAlignment.Center;
            button1.VerticalAlignment   = VerticalAlignment.Center;

            stackPanel.Children.Add(button1);

            ColorGrid colorGrid = new ColorGrid();

            colorGrid.Margin              = new Thickness(24);
            colorGrid.HorizontalAlignment = HorizontalAlignment.Center;
            colorGrid.VerticalAlignment   = VerticalAlignment.Center;

            colorGrid.SelectedColorChanged += colorGrid_SelectedColorChanged;

            stackPanel.Children.Add(colorGrid);

            Button button2 = new Button();

            button2.Content             = "테스트 버튼";
            button2.Margin              = new Thickness(24);
            button2.HorizontalAlignment = HorizontalAlignment.Center;
            button2.VerticalAlignment   = VerticalAlignment.Center;

            stackPanel.Children.Add(button2);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        [STAThread]
        private static void Main()
        {
            Application application = new Application();

            application.Run(new MainWindow());
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 색상 그리드 선택 색상 변경시 처리하기 - colorGrid_SelectedColorChanged(sender, e)

        /// <summary>
        /// 색상 그리드 선택 색상 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void colorGrid_SelectedColorChanged(object sender, EventArgs e)
        {
            ColorGrid colorGrid = sender as ColorGrid;

            Background = new SolidColorBrush(colorGrid.SelectedColor);
        }

        #endregion
    }
}